import math

import numpy as np
import pandas as pd

from coolpandas.plot import mapplot


def get_correlation(
    data_frame: pd.DataFrame, method: str = "pearson", plot: bool = True, **kwargs
) -> pd.DataFrame:
    """Get the correlations of a DataFrame.

    Args:
        data_frame (pd.DataFrame): DataFrame to get correlations from.
        method (str, optional): Correlation method. Defaults to "pearson".
        plot (bool, optional): Plot the correlation map. Defaults to True.
        **kwargs: Keyword arguments to pass to pandas.DataFrame.corr.

    Returns:
        pd.DataFrame: Correlations.
    """
    correlation_matrix: pd.DataFrame = data_frame.corr(method=method)
    truncate: callable = lambda x: math.trunc(100 * x) / 100
    correlation_matrix = correlation_matrix.applymap(truncate)
    if plot:
        mask = np.triu(np.ones_like(correlation_matrix, dtype=bool))
        mapplot(
            correlation_matrix.mask(mask),
            title=f"{method.capitalize()} correlation map",
            **kwargs,
        ).show()
    return correlation_matrix



"""DataFrames cleaning module."""
import pandas as pd


def duplicated_rows(
    data_frame: pd.DataFrame, display_summary: bool = True, drop: bool = False
) -> pd.DataFrame:
    """Get duplicated rows in a DataFrame and drop them if specified.
    Args:
        data_frame (pd.DataFrame): DataFrame to get duplicated rows.
        display_summary (bool, optional): Whether to display summary. Defaults to True.
        drop (bool, optional): Whether to drop duplicated rows. Defaults to True.
    Returns:
        pd.DataFrame: Duplicated rows.
    """
    duplicated_data_frame: pd.DataFrame = data_frame[data_frame.duplicated(keep=False)]
    if display_summary:
        print(f"Number of duplicated rows: {duplicated_data_frame.shape[0]}")
    if drop:
        data_frame.drop_duplicates(inplace=True)
    if display_summary and drop:
        print("DataFrame shape after dropping duplicated rows:")
        print(data_frame.shape)
    return duplicated_data_frame


def duplicated_columns(
    data_frame: pd.DataFrame, display_summary: bool = True, drop: bool = False
) -> pd.DataFrame:
    """Get duplicated columns in a DataFrame and drop them if specified.
    Args:
        data_frame (pd.DataFrame): DataFrame to get duplicated columns.
        display_summary (bool, optional): Whether to display summary. Defaults to True.
        drop (bool, optional): Whether to drop duplicated columns. Defaults to True.
    Returns:
        pd.DataFrame: Duplicated columns.
    """
    # uniques, indexes = np.unique(data_frame, return_index=True, axis=1)
    # return pd.DataFrame(uniq, index=data_frame.index, columns=data_frame.columns[~indexes])
    duplicates: pd.Series = data_frame.apply(lambda x: x.duplicated(), axis=1).all()
    duplicated_data_frame: pd.DataFrame = data_frame[duplicates[duplicates].index]
    if display_summary:
        print(f"Number of duplicated columns: {duplicated_data_frame.shape[1]}")
    if drop:
        data_frame.drop(columns=duplicated_data_frame.columns, inplace=True)
    if display_summary and drop:
        print("DataFrame shape after dropping duplicated columns:")
        print(data_frame.shape)
    return duplicated_data_frame



"""Barplot module."""
import pandas as pd
import plotly.express as px
import plotly.graph_objects as go

from .style import custom_template, format_title


def barplot(
    data_frame: pd.DataFrame,
    x_axis: str,
    y_axis: str,
    title: str,
    subtitle: str = None,
    **kwargs,
) -> go.Figure:
    """Create a bar plot.

    Args:
        data_frame (pd.DataFrame): DataFrame to plot.
        x_axis (str): Column to use as x axis.
        y_axis (str): Column to use as y axis.
        title (str): Title of the plot.
        subtitle (str, optional): Subtitle of the plot. Defaults to None.
        **kwargs: Keyword arguments to pass to plotly.express.bar.

    Returns:
        go.Figure: Bar plot figure.
    """
    fig = px.bar(
        data_frame,
        x=x_axis,
        y=y_axis,
        title=format_title(title, subtitle=subtitle),
        template=custom_template,
        width=800,
        height=400,
        **kwargs,
    )
    fig.update_traces(width=0.5)
    return fig



"""Get a DataFrame columns summary."""
import pandas as pd
from IPython.display import display


def get_summary(data_frame: pd.DataFrame, display_summary: bool = True) -> pd.DataFrame:
    """Get a DataFrame summary.
    Args:
        data_frame (pd.DataFrame): DataFrame to get summary.
        display_summary (bool, optional): Whether to display summary. Defaults to True.

    Returns:
        pd.DataFrame: DataFrame summary.
    """
    summary: pd.DataFrame = data_frame.dtypes.to_frame().T
    summary.rename(index={0: "dtypes"}, inplace=True)
    summary = pd.concat([summary, data_frame.describe(include="all")])
    if display_summary:
        display(summary)
    return summary
